package flowserver

import (
	"encoding/json"
	"flowserver/flowmsg"
	"log"
	"sync"

	"github.com/gorilla/websocket"
)

// FlowSession Create a session and link clients
type FlowSession struct {
	sync.Mutex
	ID string // Random handle for sessionID
	// List of clients on this session
	clients []*websocket.Conn
	Chat    ChatRoom

	RawData []byte // Just share data
}

//NewSession creates and initializes a NewSession
func NewSession(ID string) *FlowSession {
	s := &FlowSession{sync.Mutex{}, ID, []*websocket.Conn{}, ChatRoom{}, []byte{}}
	return s
}

// ClientAdd add a client to session
func (f *FlowSession) ClientAdd(c *websocket.Conn) error {
	f.Lock()
	defer f.Unlock()

	f.clients = append(f.clients, c)

	dataMap := map[string]interface{}{}
	json.Unmarshal(f.RawData, &dataMap)

	return c.WriteJSON(flowmsg.SendMessage{OP: "document", Data: dataMap})

}

// ClientRemove remove client from Session
func (f *FlowSession) ClientRemove(c *websocket.Conn) {
	f.Lock()
	defer f.Unlock()
	for i, cl := range f.clients {
		if cl == c {
			f.clients = append(f.clients[:i], f.clients[i+1:]...)
			break
		}
	}
	f.Chat.ClientRemove(c)
}

// ChatJoin the chat room on this session
func (f *FlowSession) ChatJoin(c *websocket.Conn, handle string) {
	f.Chat.ClientAdd(c, handle)
}

// DocumentUpdate client c Updates the session document
func (f *FlowSession) DocumentUpdate(c *websocket.Conn, data []byte) error {
	f.Lock()
	defer f.Unlock()
	log.Println("Document updating")

	f.RawData = data // Update
	dataMap := map[string]interface{}{}

	// contextual lock
	err := json.Unmarshal(f.RawData, &dataMap)
	if err != nil {
		return err
	}

	return f.broadcast(c, flowmsg.SendMessage{OP: "document", Data: dataMap})
}

// Document send document to client c
func (f *FlowSession) Document(c *websocket.Conn) error {
	f.Lock()
	defer f.Unlock()

	dataMap := map[string]interface{}{}
	json.Unmarshal(f.RawData, &dataMap)
	return c.WriteJSON(flowmsg.SendMessage{OP: "document", Data: dataMap})
}

// Broadcast broadcast a message in session besides C
func (f *FlowSession) Broadcast(c *websocket.Conn, v interface{}) error {
	f.Lock()
	defer f.Unlock()
	return f.broadcast(c, v)
}

//
func (f *FlowSession) broadcast(c *websocket.Conn, v interface{}) error {
	for _, sc := range f.clients {
		if sc == c { // ours
			continue
		}
		err := sc.WriteJSON(v)
		if err != nil {
			return err
		}
	}
	return nil

}
